@@ -18,7 +18,7 @@ import Data.Char (isSpace, toLower)
 import Data.Function (on)
 import Data.List as List (filter, groupBy, map, minimumBy, nub, sortBy)
 import Data.Map as Map (lookup, Map)
-import Data.Maybe (catMaybes, fromMaybe, isJust, isNothing, listToMaybe, mapMaybe)
+import Data.Maybe (catMaybes, fromMaybe, isJust, isNothing, listToMaybe, mapMaybe, maybeToList)
 import Data.Monoid ((<>))
 import Data.Set as Set (empty, fold, fromList, map, member, Set, singleton, toList, union)
 import Debian.Debianize.Prelude
@@ -38,15 +38,14 @@ import qualified Debian.Relation as D (BinPkgName(BinPkgName), Relation(..), Rel
 import Debian.Version (DebianVersion, parseDebianVersion')
 import Distribution.Compiler (CompilerFlavor(..))
 import Distribution.Package (Dependency(..), PackageIdentifier(pkgName, pkgVersion), PackageName)
-import Distribution.PackageDescription as Cabal (BuildInfo(..), BuildInfo(buildTools, extraLibs, pkgconfigDepends), Library(..), Executable(..), TestSuite(..))
-import Distribution.PackageDescription (PackageDescription)
+import Distribution.PackageDescription as Cabal (BuildInfo(..), BuildInfo(buildTools, extraLibs, pkgconfigDepends), Library(..), Executable(..), TestSuite(..), SetupBuildInfo(..), PackageDescription(setupBuildInfo))
 import qualified Distribution.PackageDescription as Cabal (PackageDescription(library, executables, testSuites))
 import Distribution.Pretty (prettyShow)
 #if MIN_VERSION_Cabal(2,0,0)
 import Distribution.Types.LegacyExeDependency (LegacyExeDependency(..))
 import Distribution.Types.PkgconfigDependency (PkgconfigDependency(..))
 #endif
-import Distribution.Version (anyVersion, asVersionIntervals, earlierVersion, foldVersionRange', fromVersionIntervals, intersectVersionRanges, isNoVersion, laterVersion, orEarlierVersion, orLaterVersion, toVersionIntervals, unionVersionRanges, VersionRange, withinVersion)
+import Distribution.Version (anyVersion, asVersionIntervals, fromVersionIntervals, intersectVersionRanges, isNoVersion, toVersionIntervals, unionVersionRanges, VersionRange, withinVersion)
 import Distribution.Version.Invert (invertVersionRange)
 import Prelude hiding (init, log, map, unlines, unlines, writeFile)
 import System.Directory (findExecutable)
@@ -79,7 +78,11 @@ unboxDependency :: Dependency_ -> Maybe Dependency
 unboxDependency (BuildDepends d) = Just d
 unboxDependency (BuildTools d) = Just d
 unboxDependency (PkgConfigDepends d) = Just d
+#if MIN_VERSION_Cabal(3,0,0)
+unboxDependency (ExtraLibs _) = Nothing -- Dependency (PackageName d) anyVersion mempty
+#else
 unboxDependency (ExtraLibs _) = Nothing -- Dependency (PackageName d) anyVersion
+#endif
 
 -- |Debian packages don't have per binary package build dependencies,
 -- so we just gather them all up here.
@@ -91,15 +94,10 @@ allBuildDepends buildInfos =
       (mergeCabalDependencies $ mapMaybe convertPkgconfig $  concatMap pkgconfigDepends buildInfos)
       (concatMap extraLibs buildInfos)
     where
-#if MIN_VERSION_Cabal(2,0,0)
       convertLegacy :: LegacyExeDependency -> Maybe Dependency
       convertLegacy = const Nothing
       convertPkgconfig :: PkgconfigDependency -> Maybe Dependency
       convertPkgconfig = const Nothing
-#else
-      convertLegacy = Just
-      convertPkgconfig = Just
-#endif
       allBuildDepends' :: Monad m => [Dependency] -> [Dependency] -> [Dependency] -> [String] -> CabalT m [Dependency_]
       allBuildDepends' buildDepends' buildTools' pkgconfigDepends' extraLibs' =
           do atoms <- get
@@ -113,14 +111,25 @@ allBuildDepends buildInfos =
           concatMap (\ cab -> fromMaybe [[D.Rel (D.BinPkgName ("lib" ++ List.map toLower cab ++ "-dev")) Nothing Nothing]]
                                         (Map.lookup cab (view (A.debInfo . D.extraLibMap) atoms)))
 
+setupBuildDepends :: SetupBuildInfo -> [Dependency_]
+setupBuildDepends = List.map BuildDepends . setupDepends
+
 -- | Take the intersection of all the dependencies on a given package name
 mergeCabalDependencies :: [Dependency] -> [Dependency]
 mergeCabalDependencies =
+#if MIN_VERSION_Cabal(3,0,0)
+    List.map (foldl1 (\ (Dependency name range1 _) (Dependency _ range2 _) -> Dependency name (intersectVersionRanges range1 range2) mempty)) . groupBy ((==) `on` dependencyPackage) . sortBy (compare `on` dependencyPackage)
+#else
     List.map (foldl1 (\ (Dependency name range1) (Dependency _ range2) -> Dependency name (intersectVersionRanges range1 range2))) . groupBy ((==) `on` dependencyPackage) . sortBy (compare `on` dependencyPackage)
+#endif
     where
+#if MIN_VERSION_Cabal(3,0,0)
+      dependencyPackage (Dependency x _ _) = x
+#else
       dependencyPackage (Dependency x _) = x
+#endif
 
--- The haskell-cdbs package contains the hlibrary.mk file with
+-- The haskell-devscripts-minimal package contains the hlibrary.mk file with
 -- the rules for building haskell packages.
 debianBuildDeps :: (MonadIO m) => PackageDescription -> CabalT m D.Relations
 debianBuildDeps pkgDesc =
@@ -128,30 +137,24 @@ debianBuildDeps pkgDesc =
        prof <- not <$> use (A.debInfo . D.noProfilingLibrary)
        let hcPackageTypes :: CompilerFlavor -> Set B.PackageType
            hcPackageTypes GHC = fromList ([B.Development] <> if prof then [B.Profiling] else [])
-#if MIN_VERSION_Cabal(1,22,0)
            hcPackageTypes GHCJS = fromList [B.Development]
-#endif
            hcPackageTypes hc = error $ "Unsupported compiler flavor: " ++ show hc
 
        let hcs = singleton hflavor -- vestigial
-       let hcTypePairs =
+           hcTypePairs =
                fold union empty $
                   Set.map (\ hc' -> Set.map (hc',) $ hcPackageTypes hc') hcs
+           setupDeps = concat . maybeToList . fmap setupBuildDepends . setupBuildInfo $ pkgDesc
 
        libDeps <- allBuildDepends (maybe [] (filter isBuildable . return . libBuildInfo) (Cabal.library pkgDesc))
        binDeps <- allBuildDepends (List.map buildInfo (filter isBuildable (Cabal.executables pkgDesc)))
        testDeps <- allBuildDepends (List.map testBuildInfo (filter isBuildable (Cabal.testSuites pkgDesc)))
-
-       -- liftIO (putStrLn ("library dependencies: " ++ show libDeps))
-       -- liftIO (putStrLn ("executable dependencies: " ++ show binDeps))
-       -- liftIO (putStrLn (intercalate "\n  " ("executables:" :  fmap show (Cabal.executables pkgDesc))))
-       -- liftIO (putStrLn ("test suite dependencies: " ++ show testDeps))
-
        testsStatus <- use (A.debInfo . D.testsStatus)
 
        cDeps <- nub . concat . concat <$> sequence
             [ mapM (buildDependencies hcTypePairs) libDeps
             , mapM (buildDependencies hcTypePairs) binDeps
+            , mapM (buildDependencies hcTypePairs) setupDeps
             , mapM (buildDependencies hcTypePairs) (if testsStatus /= D.TestsDisable then testDeps else [])
             ]
 
@@ -166,11 +169,6 @@ debianBuildDeps pkgDesc =
                         D.Rel (D.BinPkgName "haskell-devscripts") (Just $ D.GRE $ parseDebianVersion' ("0.13" :: String)) Nothing],
                        anyrel "cdbs"] ++
                       (ghcrel ++ ghcrelprof) ++
-{-
-#if MIN_VERSION_Cabal(1,22,0)
-                      (if member GHCJS (Set.map _hcFlavor hcs) then [anyrel (compilerPackageName hflavor B.Development)] else []) ++
-#endif
--}
                        bDeps ++
                        cDeps
        filterMissing xs
@@ -220,7 +218,11 @@ debianBuildDepsIndep pkgDesc =
 -- documentation package for any libraries which are build
 -- dependencies, so we have use to all the cross references.
 docDependencies :: (MonadIO m) => Dependency_ -> CabalT m D.Relations
+#if MIN_VERSION_Cabal(3,0,0)
+docDependencies (BuildDepends (Dependency name ranges _)) =
+#else
 docDependencies (BuildDepends (Dependency name ranges)) =
+#endif
     do hc <- use (A.debInfo . D.flags . compilerFlavor)
        let hcs = singleton hc -- vestigial
        omitProfDeps <- use (A.debInfo . D.omitProfVersionDeps)
@@ -231,27 +233,46 @@ docDependencies _ = return []
 -- libraries and the documentation packages, used for creating cross
 -- references.  Also the packages associated with extra libraries.
 buildDependencies :: (MonadIO m) => Set (CompilerFlavor, B.PackageType) -> Dependency_ -> CabalT m D.Relations
+#if MIN_VERSION_Cabal(3,0,0)
+buildDependencies hcTypePairs (BuildDepends (Dependency name ranges _)) =
+    use (A.debInfo . D.omitProfVersionDeps) >>= \ omitProfDeps ->
+    concat <$> mapM (\ (hc, typ) -> dependencies hc typ name ranges omitProfDeps) (toList hcTypePairs)
+#else
 buildDependencies hcTypePairs (BuildDepends (Dependency name ranges)) =
     use (A.debInfo . D.omitProfVersionDeps) >>= \ omitProfDeps ->
     concat <$> mapM (\ (hc, typ) -> dependencies hc typ name ranges omitProfDeps) (toList hcTypePairs)
+#endif
 buildDependencies _ dep@(ExtraLibs _) =
     do mp <- use (A.debInfo . D.execMap)
        return $ concat $ adapt mp dep
 buildDependencies _ dep =
     case unboxDependency dep of
+#if MIN_VERSION_Cabal(3,0,0)
+      Just (Dependency _name _ranges _) ->
+#else
       Just (Dependency _name _ranges) ->
+#endif
           do mp <- view (A.debInfo . D.execMap) <$> get
              return $ concat $ adapt mp dep
       Nothing ->
           return []
 
 adapt :: Map.Map String Relations -> Dependency_ -> [Relations]
+#if MIN_VERSION_Cabal(3,0,0)
+adapt mp (PkgConfigDepends (Dependency pkg _ _)) =
+    maybe (aptFile (unPackageName pkg)) (: []) (Map.lookup (unPackageName pkg) mp)
+adapt mp (BuildTools (Dependency pkg _ _)) =
+    maybe (aptFile (unPackageName pkg)) (: []) (Map.lookup (unPackageName pkg) mp)
+adapt _flags (ExtraLibs x) = [x]
+adapt _flags (BuildDepends (Dependency pkg _ _)) = [[[D.Rel (D.BinPkgName (unPackageName pkg)) Nothing Nothing]]]
+#else
 adapt mp (PkgConfigDepends (Dependency pkg _)) =
     maybe (aptFile (unPackageName pkg)) (: []) (Map.lookup (unPackageName pkg) mp)
 adapt mp (BuildTools (Dependency pkg _)) =
     maybe (aptFile (unPackageName pkg)) (: []) (Map.lookup (unPackageName pkg) mp)
 adapt _flags (ExtraLibs x) = [x]
 adapt _flags (BuildDepends (Dependency pkg _)) = [[[D.Rel (D.BinPkgName (unPackageName pkg)) Nothing Nothing]]]
+#endif
 
 -- There are three reasons this may not work, or may work
 -- incorrectly: (1) the build environment may be a different
@@ -305,28 +326,25 @@ dependencies hc typ name cabalRange omitProfVersionDeps =
           case isNoVersion range''' of
             True -> return Nothing
             False ->
-                Just <$> foldVersionRange'
-                          (return $ Rel' (D.Rel dname Nothing Nothing))
-                          (debianVersion' name >=> \ dv -> return $ Rel' (D.Rel dname (Just (D.EEQ dv)) Nothing))
-                          (debianVersion' name >=> \ dv -> return $ Rel' (D.Rel dname (Just (D.SGR dv)) Nothing))
-                          (debianVersion' name >=> \ dv -> return $ Rel' (D.Rel dname (Just (D.SLT dv)) Nothing))
-                          (debianVersion' name >=> \ dv -> return $ Rel' (D.Rel dname (Just (D.GRE dv)) Nothing))
-                          (debianVersion' name >=> \ dv -> return $ Rel' (D.Rel dname (Just (D.LTE dv)) Nothing))
-#if MIN_VERSION_Cabal(2,0,0)
-                          (\ x y -> debianVersion' name x >>= \ dvx ->
-                                    debianVersion' name y >>= \ dvy ->
-                                    return $ And [Rel' (D.Rel dname (Just (D.GRE dvx)) Nothing),
-                                                  Rel' (D.Rel dname (Just (D.SLT dvy)) Nothing)])
-#endif
-                          (\ x y -> debianVersion' name x >>= \ dvx ->
-                                    debianVersion' name y >>= \ dvy ->
-                                    return $ And [Rel' (D.Rel dname (Just (D.GRE dvx)) Nothing),
-                                                  Rel' (D.Rel dname (Just (D.SLT dvy)) Nothing)])
-                          (\ x y -> x >>= \ x' -> y >>= \ y' -> return $ Or [x', y'])
-                          (\ x y -> x >>= \ x' -> y >>= \ y' -> return $ And [x', y'])
-                          id
-                          range'''
+                Just <$> (cataVersionRange rangeToRange . normaliseVersionRange) range'''
           where
+            rangeToRange AnyVersionF                     = return $ Rel' (D.Rel dname Nothing Nothing)
+            rangeToRange (ThisVersionF v)                = (debianVersion' name >=> \ dv -> return $ Rel' (D.Rel dname (Just (D.EEQ dv)) Nothing)) v
+            rangeToRange (LaterVersionF v)               = (debianVersion' name >=> \ dv -> return $ Rel' (D.Rel dname (Just (D.SGR dv)) Nothing)) v
+            rangeToRange (EarlierVersionF v)             = (debianVersion' name >=> \ dv -> return $ Rel' (D.Rel dname (Just (D.SLT dv)) Nothing)) v
+            rangeToRange (OrLaterVersionF v)             = (debianVersion' name >=> \ dv -> return $ Rel' (D.Rel dname (Just (D.GRE dv)) Nothing)) v
+            rangeToRange (OrEarlierVersionF v)           = (debianVersion' name >=> \ dv -> return $ Rel' (D.Rel dname (Just (D.LTE dv)) Nothing)) v
+            rangeToRange (WildcardVersionF v)            = (\ x y -> debianVersion' name x >>= \ dvx ->
+                                    debianVersion' name y >>= \ dvy ->
+                                    return $ And [Rel' (D.Rel dname (Just (D.GRE dvx)) Nothing),
+                                                  Rel' (D.Rel dname (Just (D.SLT dvy)) Nothing)]) v (wildcardUpperBound v)
+            rangeToRange (MajorBoundVersionF v)          = (\ x y -> debianVersion' name x >>= \ dvx ->
+                                    debianVersion' name y >>= \ dvy ->
+                                    return $ And [Rel' (D.Rel dname (Just (D.GRE dvx)) Nothing),
+                                                  Rel' (D.Rel dname (Just (D.SLT dvy)) Nothing)]) v (majorUpperBound v)
+            rangeToRange (UnionVersionRangesF v1 v2)     = (\ x y -> x >>= \ x' -> y >>= \ y' -> return $ Or [x', y']) v1 v2
+            rangeToRange (IntersectVersionRangesF v1 v2) = (\ x y -> x >>= \ x' -> y >>= \ y' -> return $ And [x', y']) v1 v2
+            rangeToRange (VersionRangeParensF v)         = v
             -- Choose the simpler of the two
             range''' = canon (simpler range' range'')
             -- Unrestrict the range for versions that we know don't exist for this debian package
@@ -337,21 +355,9 @@ dependencies hc typ name cabalRange omitProfVersionDeps =
             -- a wildcard because the resulting debian version numbers have
             -- various suffixes added.
       cabalRange' | typ `elem` noVersionPackageType = anyVersion
-                  | otherwise = foldVersionRange'
-            anyVersion
-            withinVersion  -- <- Here we are turning equals into wildcard
-            laterVersion
-            earlierVersion
-            orLaterVersion
-            orEarlierVersion
-            (\ lb ub -> intersectVersionRanges (orLaterVersion lb) (earlierVersion ub))
-#if MIN_VERSION_Cabal(2,0,0)
-            (\ lb ub -> intersectVersionRanges (orLaterVersion lb) (earlierVersion ub))
-#endif
-            unionVersionRanges
-            intersectVersionRanges
-            id
-            cabalRange
+                  | otherwise = (hyloVersionRange tweak projectVersionRange . normaliseVersionRange) cabalRange
+      tweak (ThisVersionF v) = withinVersion v
+      tweak vr = embedVersionRange vr
       noVersionPackageType = (if omitProfVersionDeps then [B.Profiling] else []) ++ [B.Documentation]
       simpler v1 v2 = minimumBy (compare `on` (length . asVersionIntervals)) [v1, v2]
       -- Simplify a VersionRange
