@@ -38,19 +38,8 @@ module Debian.Debianize.Prelude
     , fromSingleton
     , (.?=)
     , escapeDebianWildcards
-#if MIN_VERSION_Cabal(2,0,0)
-    -- Cabal has its own Version type starting in 2.0.0.0
     , module Distribution.Version
     , module Distribution.Package
-#else
-    , module Data.Version
-    , mkFlagName
-    , mkPackageName
-    , mkVersion
-    , mkVersion'
-    , versionNumbers
-    , unPackageName
-#endif
     ) where
 
 
@@ -61,37 +50,32 @@ import Control.Monad (when)
 import Control.Monad.Reader (ask, ReaderT)
 import Control.Monad.State (get, MonadState, StateT, put)
 import Data.Char (isSpace)
-import Data.List as List (dropWhileEnd, intersperse, isSuffixOf, lines, map)
-import Data.Map as Map (empty, findWithDefault, foldrWithKey, fromList, insert, lookup, map, Map)
-import Data.Maybe (catMaybes, fromJust, fromMaybe, listToMaybe, mapMaybe)
+import Data.List as List (dropWhileEnd, intersperse, lines, map)
+import Data.Map (Map)
+import qualified Data.Map as Map
+import Data.Maybe (fromJust, fromMaybe, listToMaybe, mapMaybe)
 import Data.Monoid ((<>), mconcat)
 import Data.Set as Set (Set, toList)
 import qualified Data.Set as Set (findMin, fromList, null, size)
-import Data.Text as Text (lines, Text, unpack)
+import Data.Text as Text (Text, unpack)
 import Data.Text.IO (hGetContents)
-import Debian.Control (Field'(Field), lookupP, parseControl, stripWS, unControl)
+import Debian.Control (stripWS)
 import Debian.Orphans ()
 import Debian.Pretty (PP(PP))
-import qualified Debian.Relation as D (BinPkgName(BinPkgName), Relations)
+import qualified Debian.Relation as D (BinPkgName(..), Relations)
 import Debian.Relation.Common ()
 import Debian.Version (DebianVersion, parseDebianVersion', prettyDebianVersion)
-#if MIN_VERSION_Cabal(2,0,0)
 import Distribution.Package (PackageIdentifier(..), PackageName, mkPackageName, unPackageName)
 import Distribution.Version
-#else
-import Distribution.Package (PackageIdentifier(..), PackageName(..))
-import Distribution.PackageDescription (FlagName(..))
-import Data.Version
-#endif
 import Distribution.Pretty (Pretty(pretty))
 import Distribution.Verbosity (intToVerbosity, Verbosity)
 import GHC.IO.Exception (ExitCode(ExitFailure, ExitSuccess), IOErrorType(InappropriateType, NoSuchThing), IOException(IOError, ioe_description, ioe_type))
-import Prelude hiding (lookup, map)
+import Prelude hiding (map)
 import System.Directory (doesDirectoryExist, doesFileExist, getCurrentDirectory, getDirectoryContents, removeDirectory, removeFile, renameFile, setCurrentDirectory)
-import System.FilePath ((</>), dropExtension)
+import System.FilePath ((</>))
 import System.IO (hSetBinaryMode, IOMode(ReadMode), openFile, withFile)
 import System.IO.Error (catchIOError, isDoesNotExistError)
-import System.Process (readProcessWithExitCode, showCommandForUser)
+import System.Process (readProcess, readProcessWithExitCode, showCommandForUser)
 import Text.PrettyPrint.HughesPJClass as PP (text)
 
 curry3 :: ((a, b, c) -> d) -> a -> b -> c -> d
@@ -99,16 +83,12 @@ curry3 f a b c = f (a, b, c)
 
 type DebMap = Map D.BinPkgName (Maybe DebianVersion)
 
--- | Read and parse the status file for installed debian packages: @/var/lib/dpkg/status@
+-- | Query versions of installed packages
 buildDebVersionMap :: IO DebMap
 buildDebVersionMap =
-    readFile "/var/lib/dpkg/status" >>=
-    return . either (const []) unControl . parseControl "/var/lib/dpkg/status" >>=
-    mapM (\ p -> case (lookupP "Package" p, lookupP "Version" p) of
-                   (Just (Field (_, name)), Just (Field (_, version))) ->
-                       return (Just (D.BinPkgName (stripWS name), Just (parseDebianVersion' (stripWS version))))
-                   _ -> return Nothing) >>=
-    return . Map.fromList . catMaybes
+    Map.fromList . List.map lineToKV . Prelude.lines <$> readProcess "dpkg-query" ["--showformat", "${Package}\\t${Version}\\n"] ""
+    where
+        lineToKV = bimap (D.BinPkgName . stripWS) (Just . parseDebianVersion' . stripWS) . span (/= '\t')
 
 (!) :: DebMap -> D.BinPkgName -> DebianVersion
 m ! k = maybe (error ("No version number for " ++ (show . pretty . PP $ k) ++ " in " ++ show (Map.map (maybe Nothing (Just . prettyDebianVersion)) m))) id (Map.findWithDefault Nothing k m)
@@ -171,19 +151,18 @@ removeIfExists x = removeFileIfExists x >> removeDirectoryIfExists x
 dpkgFileMap :: IO (Map FilePath (Set D.BinPkgName))
 dpkgFileMap =
     do
-      let fp = "/var/lib/dpkg/info"
-      names <- getDirectoryContents fp >>= return . filter (isSuffixOf ".list")
-      let paths = List.map (fp </>) names
-      -- Read strictly to make sure we consume all the files and don't
-      -- hold tons of open file descriptors.
-      files <- mapM (strictReadF Text.lines) paths
-      return $ Map.fromList $ zip (List.map dropExtension names) (List.map (Set.fromList . List.map (D.BinPkgName . unpack)) $ files)
+      installedPackages <- Map.keys <$> buildDebVersionMap
+
+      files <- mapM (listFiles . D.unBinPkgName) installedPackages
+      return $ Map.fromList $ zip (List.map D.unBinPkgName installedPackages) (List.map (Set.fromList . List.map D.BinPkgName) $ files)
+    where
+        listFiles pkg = Prelude.lines <$> readProcess "dpkg-query" ["--listfiles", pkg] ""
 
 -- |Given a path, return the name of the package that owns it.
 debOfFile :: FilePath -> ReaderT (Map FilePath (Set D.BinPkgName)) IO (Maybe D.BinPkgName)
 debOfFile path =
     do mp <- ask
-       return $ testPath (lookup path mp)
+       return $ testPath (Map.lookup path mp)
     where
       -- testPath :: Maybe (Set FilePath) -> Maybe FilePath
       testPath Nothing = Nothing
@@ -267,12 +246,12 @@ setMapMaybe p = Set.fromList . mapMaybe p . toList
 
 zipMaps :: Ord k => (k -> Maybe a -> Maybe b -> Maybe c) -> Map k a -> Map k b -> Map k c
 zipMaps f m n =
-    foldrWithKey h (foldrWithKey g Map.empty m) n
+    Map.foldrWithKey h (Map.foldrWithKey g Map.empty m) n
     where
-      g k a r = case f k (Just a) (lookup k n) of
+      g k a r = case f k (Just a) (Map.lookup k n) of
                   Just c -> Map.insert k c r              -- Both m and n have entries for k
                   Nothing -> r                            -- Only m has an entry for k
-      h k b r = case lookup k m of
+      h k b r = case Map.lookup k m of
                   Nothing -> case f k Nothing (Just b) of
                                Just c -> Map.insert k c r -- Only n has an entry for k
                                Nothing -> r
@@ -349,11 +328,7 @@ instance Pretty (PP PackageIdentifier) where
     pretty (PP p) = pretty (PP (pkgName p)) <> text "-" <> pretty (PP (pkgVersion p))
 
 instance Pretty (PP PackageName) where
-#if MIN_VERSION_Cabal(2,0,0)
     pretty (PP p) = text (unPackageName p)
-#else
-    pretty (PP (PackageName s)) = text s
-#endif
 
 -- | Set @b@ if it currently isNothing and the argument isJust, that is
 --  1. Nothing happens if the argument isNothing
@@ -366,20 +341,3 @@ escapeDebianWildcards :: String -> String
 escapeDebianWildcards (c : more) | elem c "[]" = '\\' : c : escapeDebianWildcards more
 escapeDebianWildcards (c : more) = c : escapeDebianWildcards more
 escapeDebianWildcards "" = ""
-
-#if !MIN_VERSION_Cabal(2,0,0)
-mkFlagName :: String -> FlagName
-mkFlagName = FlagName
-mkPackageName :: String -> PackageName
-mkPackageName = PackageName
-mkVersion :: [Int] -> Version
-mkVersion ns = Version ns []
-mkVersion' :: Version -> Version
-mkVersion' = id
-versionNumbers :: Version -> [Int]
-versionNumbers (Version ns _) = ns
-#if !MIN_VERSION_Cabal(1,22,0)
-unPackageName :: PackageName -> String
-unPackageName (PackageName x) = x
-#endif
-#endif
