@@ -11,61 +11,38 @@ import Debian.Changes (ChangeLog(..), ChangeLogEntry(..))
 import Debian.Pretty (PP(PP, unPP))
 import Debian.Relation (ArchitectureReq(..), Relation(..), VersionReq(..))
 import Distribution.Compiler (CompilerId(..))
-#if MIN_VERSION_Cabal(1,22,0)
 import Distribution.Compiler (AbiTag(..))
-#endif
-#if !MIN_VERSION_Cabal(1,18,0)
-import Distribution.Compiler (CompilerFlavor(..))
-#endif
 import Distribution.License (License(..))
 import Distribution.PackageDescription (Executable(..), PackageDescription(package))
 import Distribution.Pretty (prettyShow)
 import Distribution.Simple.Compiler (Compiler(..))
-import Distribution.Version (foldVersionRange', VersionRange(..))
-#if MIN_VERSION_Cabal(2,0,0)
+import Distribution.Version (cataVersionRange, normaliseVersionRange, VersionRange(..), VersionRangeF(..))
 import Distribution.Version (Version)
-#else
-import Data.Version (showVersion, Version(..))
-#endif
 import Language.Haskell.Extension (Language(..))
-#if !MIN_VERSION_Cabal(1,21,0)
-import Language.Haskell.Extension (Extension(..), KnownExtension(..))
-#endif
 import Network.URI (URI)
 #if MIN_VERSION_hsemail(2,0,0)
 import Text.Parsec.Rfc2822 (NameAddr(..))
 #else
 import Text.ParserCombinators.Parsec.Rfc2822 (NameAddr(..))
 #endif
+import Text.PrettyPrint.HughesPJ (Doc)
 import Text.PrettyPrint.HughesPJClass (hcat, text)
 import Distribution.Pretty (Pretty(pretty))
 
 deriving instance Typeable Compiler
 deriving instance Typeable CompilerId
 
-#if MIN_VERSION_Cabal(1,22,0)
 deriving instance Typeable AbiTag
 deriving instance Data AbiTag
-#if !MIN_VERSION_Cabal(1,24,0)
-deriving instance Eq AbiTag
-#endif
 deriving instance Ord AbiTag
-#endif
 
 deriving instance Data Compiler
 deriving instance Data CompilerId
 
 deriving instance Ord Language
-#if !MIN_VERSION_Cabal(1,24,0)
-deriving instance Eq Compiler
-#endif
 deriving instance Ord Compiler
 deriving instance Ord NameAddr
 deriving instance Ord License
-#if !MIN_VERSION_Cabal(1,21,1)
-deriving instance Ord KnownExtension
-deriving instance Ord Extension
-#endif
 
 instance Ord Executable where
     compare = compare `on` exeName
@@ -91,15 +68,6 @@ deriving instance Typeable VersionReq
 deriving instance Ord ChangeLog
 deriving instance Ord ChangeLogEntry
 
-#if !MIN_VERSION_Cabal(1,18,0)
-deriving instance Data CompilerFlavor
-deriving instance Data Language
-deriving instance Data Version
-deriving instance Typeable CompilerFlavor
-deriving instance Typeable Extension
-deriving instance Typeable Language
-#endif
-
 -- Convert from license to RPM-friendly description.  The strings are
 -- taken from TagsCheck.py in the rpmlint distribution.
 instance Pretty (PP License) where
@@ -129,22 +97,20 @@ instance Pretty (PP [NameAddr]) where
     pretty = hcat . intersperse (text ", ") . map (pretty . PP) . unPP
 
 instance Pretty (PP VersionRange) where
-    pretty (PP range) =
-        foldVersionRange'
-          (text "*")
-          (\ v -> text "=" <> pretty (PP v))
-          (\ v -> text ">" <> pretty (PP v))
-          (\ v -> text "<" <> pretty (PP v))
-          (\ v -> text ">=" <> pretty (PP v))
-          (\ v -> text "<=" <> pretty (PP v))
-          (\ x _ -> text "=" <> pretty (PP x) <> text ".*") -- not exactly right
-#if MIN_VERSION_Cabal(2,0,0)
-          (\ v _ -> text " >= " <> pretty (PP v)) -- maybe this will do?
-#endif
-          (\ x y -> text "(" <> x <> text " || " <> y <> text ")")
-          (\ x y -> text "(" <> x <> text " && " <> y <> text ")")
-          (\ x -> text "(" <> x <> text ")")
-          range
+    pretty (PP range) = (cataVersionRange prettyRange . normaliseVersionRange) range
+
+prettyRange :: VersionRangeF Text.PrettyPrint.HughesPJ.Doc -> Text.PrettyPrint.HughesPJ.Doc
+prettyRange AnyVersionF                     = (text "*")
+prettyRange (ThisVersionF v)                = text "=" <> pretty (PP v)
+prettyRange (LaterVersionF v)               = text ">" <> pretty (PP v)
+prettyRange (EarlierVersionF v)             = text "<" <> pretty (PP v)
+prettyRange (OrLaterVersionF v)             = text ">=" <> pretty (PP v)
+prettyRange (OrEarlierVersionF v)           = text "<=" <> pretty (PP v)
+prettyRange (WildcardVersionF v)            = text "=" <> pretty (PP v) <> text ".*" -- not exactly right
+prettyRange (MajorBoundVersionF v)          = text " >= " <> pretty (PP v) -- maybe this will do?
+prettyRange (UnionVersionRangesF v1 v2)     = text "(" <> v1 <> text " || " <> v2 <> text ")"
+prettyRange (IntersectVersionRangesF v1 v2) = text "(" <> v1 <> text " && " <> v2 <> text ")"
+prettyRange (VersionRangeParensF v)         = text "(" <> v <> text ")"
 
 instance Pretty (PP Version) where
     pretty = text . prettyShow . unPP
